"""Frontegg proxy permissions"""

import typing
from enum import Enum
from urllib.parse import urlsplit


class _Audits(Enum):
    Read = (('GET', '/audits'),)
    Stats = (('GET', '/audits/stats'),)
    Export = (('POST', '/audits/export/pdf'), ('POST', '/audits/export/csv'))


class _Teams(Enum):
    Read = (('GET', '/team'), ('GET', '/team/roles'))
    Stats = (('GET', '/team/stats'),)
    Add = (('POST', '/team'),)
    Update = (('PUT', '/team'),)
    Delete = (('DELETE', '/team'),)
    ResendActivationEmail = (('POST', '/team/resendActivationEmail'),)
    ResetPassword = (('POST', '/team/resetPassword'),)


class FronteggPermissions(Enum):
    """Frontegg permissions enum."""
    All = (('*', '*'),)
    Audits = _Audits
    Teams = _Teams


class ForbiddenRequest(Exception):
    pass


def validate_permissions(endpoint: str, method: str, permissions: typing.List[FronteggPermissions]) -> None:
    if endpoint.startswith('http'):
        _, _, endpoint, _, _ = urlsplit(endpoint)

    if endpoint.endswith('/'):
        endpoint = endpoint.rstrip('/')

    for permission in permissions:
        if any(
                (allowed_method == '*' or method == allowed_method)
                and (allowed_endpoint == '*' or allowed_endpoint == endpoint)
                for allowed_method, allowed_endpoint in permission.value
        ):
            break
    else:
        raise ForbiddenRequest()


__all__ = ('FronteggPermissions', 'validate_permissions', 'ForbiddenRequest')



import os
from urllib.parse import urljoin


FronteggApiUrls = dict()

FronteggApiUrls['gateway'] = os.environ.get('FRONTEGG_API_GATEWAY_URL', 'https://api.frontegg.com/')

FronteggApiUrls['authentication_service'] = os.environ.get('FRONTEGG_AUTHENTICATION_SERVICE_URL',
                                                           urljoin(FronteggApiUrls['gateway'], '/auth/vendor'))

FronteggApiUrls['audits_service'] = os.environ.get('FRONTEGG_AUDITS_SERVICE_URL',
                                                   urljoin(FronteggApiUrls['gateway'], '/audits'))

FronteggApiUrls['metadata'] = os.environ.get('FRONTEGG_METADATA_SERVICE_URL',
                                             urljoin(FronteggApiUrls['gateway'], '/metadata'))

FronteggApiUrls['identity'] = os.environ.get('FRONTEGG_IDENTITY_SERVICE_URL',
                                             urljoin(FronteggApiUrls['gateway'], '/identity'))


__all__ = ('FronteggApiUrls',)



"""Frontegg client and proxy."""
from frontegg.client import FronteggRESTClient, FronteggClient, FronteggContext
from frontegg.permissions import FronteggPermissions
from frontegg.helpers import exceptions as frontegg_exceptions, frontegg_headers


__all__ = ('FronteggRESTClient', 'FronteggClient', 'FronteggPermissions', 'FronteggContext', 'frontegg_exceptions', 'frontegg_headers')



import frontegg.fastapi.frontegg as __frontegg
from fastapi import Request


def context_provider(request: Request):
    try:
        user = __frontegg.frontegg.decode_jwt(request.headers.get('Authorization'))
        return {
            'user_id': user.get('sub'),
            'tenant_id': user.get('tenantId')
        }
    except:
        return {
            'user_id': None,
            'tenant_id': None
        }




from fastapi import HTTPException
from fastapi.security.base import SecurityBase
from fastapi import Request
from typing import Optional
from frontegg.fastapi import frontegg


class GetUser(SecurityBase):
    def __init__(self, scheme_name: str = None, auto_error: bool = True):
        self.scheme_name = scheme_name or self.__class__.__name__
        self.auto_error = auto_error

    async def __call__(self, request: Request) -> Optional[str]:
        authorization: str = request.headers.get("Authorization")
        try:
            decoded = frontegg.decode_jwt(authorization)
            return decoded
        except:
            if self.auto_error:
                raise HTTPException(
                    status_code=401, detail="Unauthenticated"
                )
            else:
                return None


get_user = GetUser(auto_error=True)



from functools import wraps
import typing
from . import frontegg
from flask import abort, Response



def with_authentication(
        permission_keys: typing.Optional[list] = [],
        role_keys: typing.Optional[list] = []
):
    def decorator(f):
        @wraps(f)
        def decorated_function(*args, **kwargs):
            try:
                decoded = frontegg.decode_jwt()
                valid_permissions = all(
                    permission in decoded['permissions'] for permission in permission_keys)
                valid_roles = all(role in decoded['roles']
                                  for role in role_keys)
                if valid_permissions and valid_roles:
                    return f(*args, **kwargs)
                abort(403)
            except:
                abort(401)

        return decorated_function

    return decorator



"""A REST client for the Frontegg API."""

import typing
from abc import ABCMeta, abstractmethod
from urllib.parse import urljoin, urlparse

import arrow
import requests

from frontegg._mixins import AuditsClientMixin, SSOClientMixin
from frontegg.permissions import FronteggPermissions, validate_permissions

RequestT = typing.TypeVar('RequestT')


class FronteggContext:
    """Request context."""

    __slots__ = ('_user_id', '_tenant_id', '_permissions')

    def __init__(self,
                 user_id: str,
                 tenant_id: str,
                 permissions: typing.List[FronteggPermissions] = (FronteggPermissions.All,)) -> None:
        """

        :param user_id:
        :param tenant_id:
        :param permissions:
        """
        self._user_id = user_id
        self._tenant_id = tenant_id
        self._permissions = permissions

    @property
    def user_id(self) -> str:
        return self._user_id

    @property
    def tenant_id(self) -> str:
        return self._tenant_id

    @property
    def permissions(self) -> typing.List[FronteggPermissions]:
        return self._permissions


class BaseFronteggClient(typing.Generic[RequestT], metaclass=ABCMeta):
    """A REST client for the Frontegg API."""
    __routes_config = None

    def __init__(self,
                 client_id: str,
                 api_key: str,
                 context_callback: typing.Optional[typing.Callable[[RequestT], FronteggContext]] = None,
                 auth_middleware: typing.Callable[[RequestT], FronteggContext] = None,
                 ) -> None:
        """Initialize the Frontegg client.

        :param client_id: The client ID provided to you by Frontegg
        :param api_key: The secret key provided to you by Frontegg
        :param context_callback: A callable which accepts the current request as an argument
            and returns the frontegg context based on the request.
        """
        self.api_key = api_key
        self.client_id = client_id
        self.context_callback = context_callback
        self.auth_middleware = auth_middleware
        self.session = requests.Session()
        self._api_token = None
        self._expires_in = None

    def _maybe_refresh_api_token(self) -> None:
        """Refresh the API token if it is not present or about to expire."""
        if self._api_token is None or self._expires_in is None or arrow.utcnow() >= self._expires_in:
            self._refresh_api_token()

    def _refresh_api_token(self) -> None:
        """Refresh the API token.

        This function retrieves a new API token from the Frontegg API.

        :raises requests.HTTPError: If the Frontegg API responds with an HTTP error code, this exception is raised.
        """
        auth_response = self.session.post(self.authentication_service_url,
                                          json={
                                              'clientId': self.client_id,
                                              'secret': self.api_key
                                          })
        auth_response.raise_for_status()

        auth_response_data = auth_response.json()
        self._api_token = auth_response_data['token']
        self._expires_in = arrow.utcnow().shift(
            seconds=auth_response_data['expiresIn'] * 0.8)

    @property
    def routes_config(self):
        if self.__routes_config:
            return self.__routes_config
        response = self.request(urljoin(self.base_url, '/configs/routes'), 'GET')

        self.__routes_config = response.json()
        return self.__routes_config

    def is_public_route(self) -> bool:
        request = self.current_request
        public_routes = self.routes_config['vendorClientPublicRoutes']
        request_path = request.path.replace('/frontegg/', '')

        for route in public_routes:
            if request_path != route['url']:
                continue
            if request.method.upper() != route['method'].upper():
                continue
            if route.get('withQueryParams'):
                is_valid = True
                for query_param in route['withQueryParams']:
                    value = request.args.get(query_param['key'])
                    if not value:
                        is_valid = False
                        break
                    if query_param['value'] and value != query_param['value']:
                        is_valid = False
                        break

                if not is_valid:
                    continue
            return True
        return False




    @property
    @abstractmethod
    def base_url(self) -> str:
        pass

    @property
    @abstractmethod
    def authentication_service_url(self) -> str:
        pass

    @property
    def api_token(self) -> str:
        """The API Token to use to authenticate against Frontegg's services.

        :return: The API token.
        :raises requests.HTTPError: If the Frontegg API responds with an HTTP error code, this exception is raised.
        """
        self._maybe_refresh_api_token()
        return self._api_token

    @property
    @abstractmethod
    def current_request(self) -> RequestT:
        pass

    @property
    def context(self) -> typing.Optional[FronteggContext]:
        """The context under which the request is proxied.

        :return: The :class:`FronteggContext` result from the callback.
        """
        context = None
        if self.context_callback:
            context = self.context_callback(self.current_request)

        return context

    def request(self,
                endpoint: str,
                method: str,
                json: typing.Optional[dict] = None,
                params: typing.Optional[dict] = None,
                tenant_id: typing.Optional[str] = None,
                host: typing.Optional[str] = None,
                headers: typing.Optional[dict] = {}) -> requests.Response:
        """Perform a request to Frontegg's API.

        :param endpoint: The endpoint to perform the request to.
        :param method: The HTTP method to use while performing the request.
        :param data: When performing a GET request, the query string to pass as part of the request, if applicable..
            In any other request, the JSON payload to pass as the body of the request, if applicable.
        :param tenant_id: Override the tenant id from the context.
        :return: The response of the request.
        :raises requests.HTTPError: If the Frontegg API responds with an HTTP error code, this exception is raised.
        """
        context = self.context
        if tenant_id:
            user_id = None
        else:
            if context:
                user_id, tenant_id = context.user_id, context.tenant_id
            else:
                user_id, tenant_id = None, None

        if context:
            permissions = context.permissions
        else:
            permissions = (FronteggPermissions.All,)
        validate_permissions(endpoint, method, permissions=permissions)

        newHeaders = {}
        if headers.get('Cookie'):
            newHeaders['Cookie'] = headers['Cookie']
        newHeaders['x-access-token'] = self.api_token
        newHeaders['frontegg-vendor-host'] = host

        if tenant_id:
            newHeaders['frontegg-tenant-id'] = tenant_id
        if user_id:
            newHeaders['frontegg-user-id'] = user_id

        return self.session.request(
            method,
            urljoin(self.base_url, endpoint),
            allow_redirects=False,
            headers=newHeaders,
            json=json,
            params=params)


class FronteggRESTClient(BaseFronteggClient[None]):
    """A standalone Frontegg REST client."""

    def __init__(self,
                 client_id: str,
                 api_key: str,
                 context_callback: typing.Optional[typing.Callable[[
                     RequestT], FronteggContext]] = None,
                 base_url: str = 'https://api.frontegg.com/',
                 authentication_service_url: typing.Optional[str] = None) -> None:
        """Initialize the Frontegg REST client.

        :param client_id: The client ID provided to you by Frontegg.
        :param api_key: The secret key provided to you by Frontegg.
        :param context_callback: A callable which returns the user id and tenant id based on the current request.
        :param base_url: The base URL of the API.
        :param authentication_service_url: The URL for authenticating against Frontegg.
        """
        super().__init__(client_id, api_key, context_callback)
        self._base_url = base_url

        if authentication_service_url:
            self._authentication_service_url = authentication_service_url
        else:
            self._authentication_service_url = urljoin(
                base_url, '/vendors/auth/token')

    @property
    def base_url(self) -> str:
        return self._base_url

    @property
    def authentication_service_url(self) -> str:
        return self._authentication_service_url

    @property
    def current_request(self):
        return None


class FronteggClient(SSOClientMixin, AuditsClientMixin):
    def __init__(self,
                 client_id: str,
                 api_key: str,
                 context_callback: typing.Optional[typing.Callable[[
                     RequestT], FronteggContext]] = None,
                 base_url: str = 'https://api.frontegg.com/',
                 authentication_service_url: typing.Optional[str] = None,
                 audits_service_url: typing.Optional[str] = None,
                 metadata_service_url: typing.Optional[str] = None,
                 team_service_url: typing.Optional[str] = None) -> None:
        """Initialize the Frontegg client.

        :param client_id: The client ID provided to you by Frontegg.
        :param api_key: The secret key provided to you by Frontegg.
        :param context_callback: A callable which returns the user id and tenant id based on the current request.
        :param base_url: The base URL of the API.
        :param authentication_service_url: The URL for authenticating against Frontegg.
        """
        self._client = FronteggRESTClient(
            client_id, api_key, context_callback, base_url, authentication_service_url)

        if audits_service_url:
            self._audits_service_url = audits_service_url
        else:
            self._audits_service_url = urljoin(base_url, '/audits')

        if metadata_service_url:
            self._metadata_service_url = metadata_service_url
        else:
            self._metadata_service_url = urljoin(base_url, '/metadata')

        if team_service_url:
            self._team_service_url = team_service_url
        else:
            self._team_service_url = urljoin(base_url, '/team')

    @property
    def _config(self) -> dict:
        return {
            'FRONTEGG_AUDITS_SERVICE_URL': self._audits_service_url,
            'FRONTEGG_METADATA_SERVICE_URL': self._metadata_service_url,
            'FRONTEGG_TEAM_SERVICE_URL': self._team_service_url
        }


__all__ = ('BaseFronteggClient', 'FronteggRESTClient', 'FronteggClient')



from frontegg import FronteggContext
import frontegg.flask



def context_provider(req):
    decoded = frontegg.flask.frontegg.decode_jwt(verify=False)
    return FronteggContext(decoded.get('sub'), decoded.get('tenantId'))



from .flask import frontegg
from .withAuthentication import with_authentication
from .contextProvider import context_provider
from .authentication_middleware import authentication_middleware
__all__ = ('frontegg', 'with_authentication', 'context_provider', 'authentication_middleware')



from frontegg.baseConfig.frontegg_authenticator import FronteggAuthenticator
from frontegg.helpers.frontegg_urls import frontegg_urls
from urllib.parse import urljoin
import typing
from frontegg.helpers.frontegg_headers import frontegg_headers
from frontegg.helpers.exceptions import HttpException
from .identity_mixin import IdentityClientMixin


class FronteggProxy(FronteggAuthenticator, IdentityClientMixin):
    __routes_config = None

    def proxy_request(self, request, method: str, path: str, host: str, params: dict,
                      body: typing.Optional[any], cookies: typing.Optional[dict] = None,
                      headers: typing.Optional[dict] = {}):
        if not request:
            raise Exception('request is required')

        if not method:
            raise Exception('method is required')

        if not path:
            raise Exception('path is required')

        path_without_frontegg = path.replace(self.middleware_prefix, '').replace('/'+self.middleware_prefix, '')

        public_route = self.is_public_route(path_without_frontegg, params, method)

        if self.authentication_middleware is not None and not public_route:
            try:
                self.authentication_middleware(request)
            except HttpException as response:
                return response
            except:
                return HttpException('Something went wrong', 500)

        url = urljoin(frontegg_urls.base_url, path_without_frontegg)
        headers = self.clean_headers(headers, host)
        headers = self.set_context(headers, request)

        if self.should_refresh_vendor_token:
            self.refresh_vendor_token()

        response = self.vendor_session_request.request(
            method,
            url,
            headers=headers,
            cookies=cookies,
            data=body,
            params=params
        )

        return response

    def clean_headers(self, headers: dict, host: str) -> dict:
        new_headers = dict()
        ignored_headers = ['host', 'authorization']
        for key, value in headers.items():
            if 'access-control' not in key.lower() and key.lower() not in ignored_headers:
                new_headers[key] = value

        new_headers[frontegg_headers['vendor_host']] = host
        return new_headers

    def set_context(self, headers: dict, request) -> dict:
        context = self.context_callback(request)
        if context.get('tenant_id'):
            headers[frontegg_headers['tenant_id']] = context['tenant_id']
        if context.get('user_id'):
            headers[frontegg_headers['user_id']] = context['user_id']

        return headers

    @property
    def routes_config(self):
        if self.__routes_config:
            return self.__routes_config
        response = self.vendor_session_request.get(frontegg_urls.routes_config)

        self.__routes_config = response.json()
        return self.__routes_config

    def is_public_route(self, path: str, params: dict, method: str) -> bool:
        public_routes = self.routes_config['vendorClientPublicRoutes']

        for route in public_routes:
            if path != route['url']:
                continue
            if method != route['method'].upper():
                continue
            if route.get('withQueryParams'):
                is_valid = True
                for query_param in route['withQueryParams']:
                    value = params.get(query_param['key'])
                    if not value:
                        is_valid = False
                        break
                    if query_param['value'] and value != query_param['value']:
                        is_valid = False
                        break

                if not is_valid:
                    continue
            return True
        return False



from fastapi import Request
from frontegg.helpers.exceptions import UnauthenticatedException
import frontegg.fastapi.frontegg as __frontegg


def authentication_middleware(request: Request):
    try:
        __frontegg.frontegg.decode_jwt(request.headers.get('Authorization'))
        return None
    except:
        raise UnauthenticatedException()



from frontegg.baseConfig.frontegg_config import FronteggConfig
import requests
from frontegg.helpers.frontegg_urls import frontegg_urls
import arrow


class FronteggAuthenticator(FronteggConfig):
    __access_token = None
    __access_token_expiration = None
    vendor_session_request = requests.session()

    def __init__(self, client_id: str, api_key, context_provider=None, authentication_middleware=None, middleware_prefix = None):
        super(FronteggAuthenticator, self).__init__(client_id, api_key, context_provider, authentication_middleware, middleware_prefix=middleware_prefix)
        self.refresh_vendor_token()

    @property
    def should_refresh_vendor_token(self) -> bool:
        return self.__access_token is None or self.__access_token_expiration is None or arrow.utcnow() >= self.__access_token_expiration

    def refresh_vendor_token(self) -> None:
        body = {
            'clientId': self.client_id,
            'secret': self.api_key
        }
        auth_url = frontegg_urls.authentication_service['authenticate_vendor']
        auth_response = self.vendor_session_request.post(auth_url, json=body)

        auth_response.raise_for_status()
        response_body = auth_response.json()
        self.__access_token = response_body['token']
        self.__access_token_expiration = arrow.utcnow().shift(
            seconds=response_body['expiresIn'] * 0.8)

        self.vendor_session_request.headers.update({'x-access-token': self.__access_token})




class HttpException(Exception):
    def __init__(self, content, status_code: int, headers={}):
        self.status_code = status_code
        self.content = content
        self.headers = headers


class UnauthenticatedException(HttpException):
    def __init__(self, content='Unauthenticated', status_code: int = 401, headers={}):
        super(UnauthenticatedException, self).__init__(content, status_code)


class UnauthorizedException(HttpException):
    def __init__(self, content='Unauthorized', status_code: int = 403, headers={}):
        super(UnauthorizedException, self).__init__(content, status_code)



import typing


class FronteggConfig():
    client_id: str = None
    api_key: str = None

    def __init__(self, client_id: str, api_key, context_provider: typing.Callable = None,
                 authentication_middleware=None, middleware_prefix: str = None):
        if client_id is None:
            raise Exception('client_id is required')
        if api_key is None:
            raise Exception('api_key is required')
        if context_provider is None:
            raise Exception('context_provider is required')

        self.client_id = client_id
        self.api_key = api_key
        self.context_callback = context_provider
        self.authentication_middleware = authentication_middleware
        self.middleware_prefix = middleware_prefix or 'frontegg/'



import typing
from abc import ABCMeta, abstractmethod
from urllib.parse import urlsplit, urlunsplit
import jwt
from flask import request


def _get_filters(count, filter, filters, offset, sort_by, sort_direction):
    if not filters:
        filters = {}
    data = filters
    if filter:
        data['filter'] = filter
    if sort_by:
        data['sortBy'] = sort_by
    if sort_direction:
        data['sortDirection'] = sort_direction
    if offset:
        data['offset'] = offset
    if count:
        data['count'] = count
    return data


class AuditsClientMixin(metaclass=ABCMeta):
    @property
    @abstractmethod
    def _config(self) -> dict:
        """A dictionary containing the configuration for Frontegg."""
        pass

    def get_audits(self,
                   tenant_id: typing.Optional[str] = None,
                   filter: typing.Optional[str] = None,
                   sort_by: typing.Optional[str] = None,
                   sort_direction: typing.Optional[str] = None,
                   offset: typing.Optional[int] = None,
                   count: typing.Optional[int] = None,
                   filters: typing.Optional[dict] = None) -> dict:
        """Fetch audit records from the Frontegg API.

        TODO: Document filters and other options with examples.

        :param tenant_id:
        :param filter:
        :param sort_by:
        :param sort_direction:
        :param offset:
        :param count:
        :param filters:
        :raises requests.HTTPError: If the Frontegg API responds with an HTTP error code, this exception is raised.
        :return: A dictionary representing the parsed JSON response.
        """
        data = _get_filters(count, filter, filters, offset,
                            sort_by, sort_direction)

        response = self._client.request(
            self._config['FRONTEGG_AUDITS_SERVICE_URL'],
            'GET',
            params=data,
            tenant_id=tenant_id
        )
        response.raise_for_status()

        return response.json()

    def send_audits(self, audits: dict, tenantId) -> dict:
        """Create new audits.

        :param audits: A dictionary containing the audit.
        :raises requests.HTTPError: If the Frontegg API responds with an HTTP error code, this exception is raised.
        :return: The newly created audit object.
        """
        response = self._client.request(
            self._config['FRONTEGG_AUDITS_SERVICE_URL'],
            'POST',
            json=audits,
            tenant_id=tenantId
        )
        response.raise_for_status()

        return response.json()

    def get_audit_stats(self, tenant_id: str) -> dict:
        """Fetch audit statistics.

        :param tenant_id: The tenant id to fetch the stats for.
        :raises requests.HTTPError: If the Frontegg API responds with an HTTP error code, this exception is raised.
        :return: The audit statistics.
        """
        scheme, network, location, path, query = urlsplit(
            self._config['FRONTEGG_AUDITS_SERVICE_URL'])
        if location.endswith('/'):
            location += 'stats/'
        else:
            location += '/stats/'
        endpoint = urlunsplit((scheme, network, location, path, query))
        response = self._client.request(
            endpoint,
            'GET',
            json={
                'tenantId': tenant_id
            },
            tenant_id=tenant_id
        )
        response.raise_for_status()

        return response.json()

    def get_audits_metadata(self) -> dict:
        """Retrieve metadata for audits.

        :raises requests.HTTPError: If the Frontegg API responds with an HTTP error code, this exception is raised.
        :return The metadata for audits.
        """
        response = self._client.request(self._config['FRONTEGG_METADATA_SERVICE_URL'],
                                        'GET',
                                        params={'entityName': 'audits'})
        response.raise_for_status()

        return response.json()

    def set_audits_metadata(self, metadata: dict) -> dict:
        """Set the metadata for audits.

        :param metadata: The new metadata for audits.
        :raises requests.HTTPError: If the Frontegg API responds with an HTTP error code, this exception is raised.
        :return: The newly set metadata for audits.
        """
        response = self._client.request(self._config['FRONTEGG_METADATA_SERVICE_URL'],
                                        'POST',
                                        json=metadata)
        response.raise_for_status()

        return response.json()

    def export_pdf(self,
                   tenant_id: typing.Optional[str] = None,
                   filter: typing.Optional[str] = None,
                   sort_by: typing.Optional[str] = None,
                   sort_direction: typing.Optional[str] = None,
                   offset: typing.Optional[int] = None,
                   count: typing.Optional[int] = None,
                   filters: typing.Optional[dict] = None) -> bytes:
        """Export audit records from the Frontegg API to PDF.

        TODO: Document filters and other options with examples.

        :param tenant_id:
        :param filter:
        :param sort_by:
        :param sort_direction:
        :param offset:
        :param count:
        :param filters:
        :raises requests.HTTPError: If the Frontegg API responds with an HTTP error code, this exception is raised.
        :return: The PDF as a bytes string.
        """
        scheme, network, location, path, query = urlsplit(
            self._config['FRONTEGG_AUDITS_SERVICE_URL'])
        if location.endswith('/'):
            location += 'export/pdf'
        else:
            location += '/export/pdf'
        endpoint = urlunsplit((scheme, network, location, path, query))

        data = _get_filters(count, filter, filters, offset,
                            sort_by, sort_direction)

        response = self._client.request(
            endpoint,
            'get',
            data=data,
            tenant_id=tenant_id
        )
        response.raise_for_status()

        return response.content

    def export_csv(self,
                   tenant_id: typing.Optional[str] = None,
                   filter: typing.Optional[str] = None,
                   sort_by: typing.Optional[str] = None,
                   sort_direction: typing.Optional[str] = None,
                   offset: typing.Optional[int] = None,
                   count: typing.Optional[int] = None,
                   filters: typing.Optional[dict] = None) -> bytes:
        """Export audit records from the Frontegg API to PDF.

        TODO: Document filters and other options with examples.

        :param tenant_id:
        :param filter:
        :param sort_by:
        :param sort_direction:
        :param offset:
        :param count:
        :param filters:
        :raises requests.HTTPError: If the Frontegg API responds with an HTTP error code, this exception is raised.
        :return: The PDF as a bytes string.
        """
        scheme, network, location, path, query = urlsplit(
            self._config['FRONTEGG_AUDITS_SERVICE_URL'])
        if location.endswith('/'):
            location += 'export/csv'
        else:
            location += '/export/csv'
        endpoint = urlunsplit((scheme, network, location, path, query))

        data = _get_filters(count, filter, filters, offset,
                            sort_by, sort_direction)

        response = self._client.request(
            endpoint,
            'get',
            data=data,
            tenant_id=tenant_id
        )
        response.raise_for_status()

        return response.content


class SSOClientMixin(metaclass=ABCMeta):
    @property
    @abstractmethod
    def _config(self) -> dict:
        """A dictionary containing the configuration for Frontegg."""
        pass

    def prelogin(self, payload: str) -> str:
        """
        Perform SSO prelogin phase

        Parameters:
        payload: can be email of tenantId
        """
        response = self._client.request(
            '/'.join([self._config['FRONTEGG_TEAM_SERVICE_URL'],
                      'resources/sso/v1/prelogin']),
            'POST',
            json={'payload': payload}
        )
        return response

    def postlogin(self, saml_response):
        response = self._client.request(
            '/'.join([self._config['FRONTEGG_TEAM_SERVICE_URL'],
                      'resources/sso/v1/postlogin']),
            'POST',
            json=saml_response
        )
        return response.json()


class IdentityClientMixin(metaclass=ABCMeta):
    __publicKey = None

    @property
    @abstractmethod
    def _config(self) -> dict:
        """A dictionary containing the configuration for Frontegg."""
        pass

    def getPublicKey(self) -> str:
        if self.__publicKey:
            return self.__publicKey
        url = '/'.join([self._config['FRONTEGG_IDENTITY_SERVICE_URL'],
                        'resources/configurations/v1'])
        response = self._client.request(
            url,
            'GET'
        )
        data = response.json()
        self.__publicKey = data.get('publicKey')
        return self.__publicKey


    def decode_jwt(self, verify: typing.Optional[bool] = True):
        authorization_header = request.headers.get('Authorization')
        if not authorization_header:
            raise Exception('Authorization headers is missing')
        jwt_token = authorization_header.replace('Bearer ', '')
        if verify:
            public_key = self.getPublicKey()
            decoded = jwt.decode(jwt_token, public_key, algorithms='RS256')
        else:
            decoded = jwt.decode(jwt_token, algorithms='RS256', verify = False)
        request.user = decoded
        return decoded




from .get_user import get_user, GetUser
from .context_provider import context_provider
from .authentication_middleware import authentication_middleware

__all__ = ('get_user', 'GetUser', 'context_provider', 'authentication_middleware')


from abc import ABCMeta, abstractmethod
from frontegg.helpers.frontegg_urls import frontegg_urls
import typing
import jwt
import requests


class IdentityClientMixin(metaclass=ABCMeta):
    __publicKey = None

    @property
    @abstractmethod
    def vendor_session_request(self) -> requests.Session:
        pass

    def get_public_key(self) -> str:
        if self.__publicKey:
            return self.__publicKey

        response = self.vendor_session_request.get(frontegg_urls.identity_service['vendor_config'])
        data = response.json()
        self.__publicKey = data.get('publicKey')
        return self.__publicKey

    def decode_jwt(self, authorization_header, verify: typing.Optional[bool] = True):
        if not authorization_header:
            raise Exception('Authorization headers is missing')
        jwt_token = authorization_header.replace('Bearer ', '')
        if verify:
            public_key = self.get_public_key()
            decoded = jwt.decode(jwt_token, public_key, algorithms='RS256')
        else:
            decoded = jwt.decode(jwt_token, algorithms='RS256', verify=False)
        return decoded



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['frontegg',
 'frontegg.baseConfig',
 'frontegg.fastapi',
 'frontegg.fastapi.secure_access',
 'frontegg.flask',
 'frontegg.helpers']

package_data = \
{'': ['*']}

install_requires = \
['arrow>=0.15.5,<0.16.0',
 'cryptography>=3.1,<4.0',
 'pyjwt>=1.7.1,<2.0.0',
 'requests>=2.22.0,<3.0.0']

extras_require = \
{u'flask': ['flask>=1.0,<2.0']}

setup_kwargs = {
    'name': 'frontegg',
    'version': '0.1.24',
    'description': 'Frontegg is a web platform where SaaS companies can set up their fully managed, scalable and brand aware - SaaS features and integrate them into their SaaS portals in up to 5 lines of code.',
    'long_description': '.. image:: https://fronteggstuff.blob.core.windows.net/frongegg-logos/logo-transparent.png\n   :alt: Frontegg\n\nFrontegg is a web platform where SaaS companies can set up their fully managed, scalable and brand aware - SaaS features and integrate them into their SaaS portals in up to 5 lines of code.\n\nInstallation\n------------\n\nYou can install this library by using pip::\n\n    pip install frontegg\n\nIf you\'d like to use Frontegg in your flask project use the flask extra dependency::\n\n    pip install frontegg[flask]\n\nUsage\n-----\n\nYou can directly use the Frontegg REST client to access your data when you need to.\nThe raw client directly returns a `requests.Response`.\n\nThis is useful in Python scripts or in cases where you need to access the data from the server side:\n\n.. code-block:: pycon\n\n    >>> from frontegg import FronteggRESTClient\n    >>> client = FronteggRESTClient(\'your-client-id\', \'your-client-secret\')\n    >>> response = client.request(\'/metadata\', \'GET\', params={\'entityName\': \'audits\'})\n    >>> print(response.json())\n    {\'rows\': [{\'_id\': \'5d3d2ee54a04a50033da91df\', \'entityName\': \'audits\', \'properties\': [\n        {\'_id\': \'5d3d2ee54a04a50033da91e6\', \'name\': \'createdAt\', \'displayName\': \'Date\', \'type\': \'Timestamp\',\n         \'filterable\': True, \'sortable\': True},\n        {\'_id\': \'5d3d2ee54a04a50033da91e5\', \'name\': \'user\', \'displayName\': \'User\', \'type\': \'UserIdentity\',\n         \'filterable\': True, \'sortable\': True},\n        {\'_id\': \'5d3d2ee54a04a50033da91e4\', \'name\': \'resource\', \'displayName\': \'Resource\', \'type\': \'AlphaNumeric\',\n         \'filterable\': True, \'sortable\': True},\n        {\'_id\': \'5d3d2ee54a04a50033da91e3\', \'name\': \'action\', \'displayName\': \'Action\', \'type\': \'AlphaNumeric\',\n         \'filterable\': True, \'sortable\': True}, {\n            \'_id\': \'5d3d2ee54a04a50033da91e2\', \'name\': \'severity\', \'displayName\': \'Severity\', \'type\': \'AlphaNumeric\',\n            \'filterable\': True, \'sortable\': True},\n        {\'_id\': \'5d3d2ee54a04a50033da91e1\', \'name\': \'ip\', \'displayName\': \'IP Address\', \'type\': \'AlphaNumeric\',\n         \'filterable\': True, \'sortable\': True},\n        {\'_id\': \'5d3d2ee54a04a50033da91e0\', \'name\': \'message\', \'displayName\': \'Message\', \'type\': \'AlphaNumeric\',\n         \'filterable\': True, \'sortable\': False}], \'vendorId\': \'my-client-id\',\n                          \'id\': \'39372f0f-1d14-4ecd-8462-1b22d5ca9264\', \'createdAt\': \'2019-07-28T05:13:09.723Z\',\n                          \'updatedAt\': \'2019-07-28T05:13:09.723Z\', \'__v\': 0}]}\n\nYou can pass a context callback to provide the user id and the tenant id that are relevant to the request:\n\n.. code-block:: pycon\n\n    >>> from frontegg import FronteggRESTClient, FronteggContext\n    >>> client = FronteggRESTClient(\'your-client-id\', \'your-client-secret\', context_callback=lambda request: FronteggContext(\'user_id@user.com\', \'tenant-id\'))\n    >>> response = client.request(\'/audits/stats\', \'GET\')\n    >>> print(response.json())\n    {\'totalToday\': 2, \'severeThisWeek\': 0}\n\nHigher Level Client\n~~~~~~~~~~~~~~~~~~~\n\nYou can access the audits API directly using the `FronteggClient`:\n\n.. code-block:: pycon\n\n    >>> from frontegg import FronteggClient\n    >>> client = FronteggClient(\'your-client-id\', \'your-client-secret\')\n    >>> client.send_audits({"username": "test", "severity": "Info"}, tenantId=\'my-tenant-id\')\n\nProxying Requests From Other Clients\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\nThe browser and possibly other clients in your architecture need to access Frontegg from your proxy\nas it determines the context of the request.\n\nThis package currently only supports doing so from Flask. In the future, more frameworks may be added.\n\nFlask\n+++++\n\nTo proxy requests to Frontegg in your Flask application the following configuration must be provided:\n\n.. code-block:: python\n\n    from flask import Flask\n    app = Flask(\'example\')\n    app.config[\'FRONTEGG_CLIENT_ID\'] = \'your client id\'\n    app.config[\'FRONTEGG_API_KEY\'] = \'your api key\'\n\nYou can optionally provide the context callback as a configuration setting as well:\n\n.. code-block:: python\n\n    from frontegg import FronteggContext\n    app.config[\'FRONTEGG_CONTEXT_RESOLVER\'] = lambda request: FronteggContext(\'user_id@user.com\', \'my-tenant-id\')\n\nThe request argument will be filled with the `flask.request` object.\nYou can use it to determine the user id and the tenant id.\n\nIn addition, different users may or may not access specific data and thus have different permissions.\nYou can use the `frontegg.FronteggPermissions` enum to limit access to your data.\n\n.. code-block:: python\n\n    from frontegg import FronteggContext, FronteggPermissions\n\n    def context_resolver(request):\n        if is_admin_user(request):\n            permissions = (FronteggPermissions.All,)\n        else:\n            permissions = (FronteggPermissions.Teams.value.Read,)\n\n        return FronteggContext(\'user_id@user.com\', \'my-tenant-id\', permissions=permissions)\n\n\nTo begin proxying requests you should\n\n.. code-block:: python\n\n    from flask import Flask\n    from frontegg import FronteggContext\n    from frontegg.flask import frotnegg\n    app = Flask(\'example\')\n    app.config[\'FRONTEGG_CLIENT_ID\'] = \'your client id\'\n    app.config[\'FRONTEGG_API_KEY\'] = \'your api key\'\n    app.config[\'FRONTEGG_CONTEXT_RESOLVER\'] = lambda request: FronteggContext(\'user_id@user.com\', \'my-tenant-id\')\n    frontegg.init_app(app)\n\n\naiohttp async support\n+++++++++++++++++++++\n\nIn case you are using aiohttp as your web infrastructure, Frontegg has support for that as well.\nFor adding the frontegg middleware to the aiohttp you should\n\n.. code-block:: python\n\n    from aiohttp import web\n    from frontegg.aiohttp import Frontegg\n\n    app = web.Application()\n    Frontegg(app, \'my-client-id\', \'my-api-key\', context_resolver)\n\n',
    'author': 'Frontegg LTD',
    'author_email': 'hello@frontegg.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://frontegg.com/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from fastapi import FastAPI, Request, Response
from frontegg.baseConfig.frontegg_proxy import FronteggProxy
import typing
import frontegg.fastapi.secure_access as secure_access


class Frontegg(FronteggProxy):
    def __init__(self):
        pass

    def init_app(
            self,
            app: FastAPI,
            client_id: str,
            api_key,
            context_provider: typing.Callable = None,
            authentication_middleware=None,
            with_secure_access: bool = False,
            middleware_prefix: str = None,
    ):

        if with_secure_access:
            context_provider = context_provider or secure_access.context_provider
            authentication_middleware = authentication_middleware or secure_access.authentication_middleware

        super(Frontegg, self).__init__(client_id, api_key, context_provider, authentication_middleware, middleware_prefix)

        @app.middleware('http')
        async def middleware(request: Request, call_next):
            path = request.url.path
            if path.startswith(self.middleware_prefix) or path.startswith('/'+self.middleware_prefix):
                body = await request.body()
                response = await self.proxy_request(request=request, method=request.method, path=path,
                                              host=request.client.host, body=body, headers=request.headers,
                                              params=request.query_params)
                return Response(content=response.content, status_code=response.status_code, headers=response.headers)
            else:
                response = await call_next(request)
                return response


frontegg = Frontegg()
